using Entities;
using Model;
using Shared;

namespace EFMappers;

public static class ActivityMapper
{
    private static GenericMapper<Activity, ActivityEntity> _mapper = new GenericMapper<Activity, ActivityEntity>();
    // ! RESET
    // ? Quand on fait appel au reset ?
    // * Apres des saves changing ou rollback.
    public static void Reset()
    {
        _mapper.Reset();
    }
    public static Activity? ToModel(this ActivityEntity entity)
    {
        // return entity.ToModel();
        return entity.ToT(_mapper, activity => new Activity (
            entity.IdActivity,
            entity.Type,
            new DateTime(entity.Date.Year, entity.Date.Month, entity.Date.Day),
            new DateTime().Add(entity.StartTime.ToTimeSpan()),
            new DateTime().Add(entity.EndTime.ToTimeSpan()),
            entity.EffortFelt,
            entity.Variability,
            entity.Variance,
            entity.StandardDeviation,
            entity.Average,
            entity.Maximum,
            entity.Minimum,
            entity.AverageTemperature,
            entity.HasAutoPause)
            // ! regarder a ce que le model est bien les relation comme l'EF
        , (activity, entity) => activity.heartRates = entity.HeartRates.Select(hr => hr.ToModel()).ToList());

    }
    // dictionnaire;


    public static ActivityEntity? ToEntity(this Activity model)
    {

        Func<Activity,ActivityEntity> create = activity => new ActivityEntity
        {
            IdActivity = model.Id,
            Type = model.Type,
            Date = DateOnly.FromDateTime(model.Date),
            StartTime = TimeOnly.FromDateTime(model.StartTime),
            EndTime = TimeOnly.FromDateTime(model.EndTime),
            EffortFelt = model.Effort,
            Variability = model.Variability,
            Variance = model.Variance,
            StandardDeviation = model.StandardDeviation,
            Average = model.Average,
            Maximum = model.Maximum,
            Minimum = model.Minimum,
            AverageTemperature = model.AverageTemperature,
            HasAutoPause = model.HasAutoPause
        };

        Action<Activity,ActivityEntity> link = (activity, entity) =>
 {
entity.HeartRates = 
// Juste faire un mappeur pour les heartRate et du coup du fait cette appel 
// activity.heartRates.Select(hr => hr.ToEntity()).ToList(); 
activity.heartRates.Select(hr => new HeartRateEntity
            {
                
                IdHeartRate = hr.Id,
                Altitude = hr.Altitude,
                Time = hr.Time,
                Temperature = hr.Temperature,
                Bpm = hr.Bpm,
                Longitude = hr.Longitude,
                Latitude = hr.Latitude,
                ActivityId = entity.IdActivity
            }).ToList<HeartRateEntity>();
            entity.DataSource = activity.DataSource 
            // entity.DataSource = activity.DataSource.ToEntity();
            // entity.Athlete = activity.Athlete.ToEntity();
 };       
        

        // return model.ToEntity();
        return model.ToU(_mapper, create,  );
            // ! regarder a ce que le model est bien les relation comme l'EF

        /*(activity, entity) => entity.HeartRates = activity.heartRates.Select(hr => new HeartRateEntity
            {
                IdHeartRate = hr.Id,
                Altitude = hr.Altitude,
                Time = hr.Time,
                Temperature = hr.Temperature,
                Bpm = hr.Bpm,
                Longitude = hr.Longitude,
                Latitude = hr.Latitude,
                ActivityId = entity.IdActivity
            }).ToList<HeartRateEntity>()*/
    }

    public static IEnumerable<Activity?> ToModels(this IEnumerable<ActivityEntity> entities)
        => entities.Select(a => a.ToModel());

    public static IEnumerable<ActivityEntity?> ToEntities(this IEnumerable<Activity> models)
        => models.Select(a => a.ToEntity());
}